@@ -15,7 +15,7 @@ static void dbarr_print(migratorContext *ctx, DbInfoArr *arr,
 			Cluster whichCluster);
 static void relarr_print(migratorContext *ctx, RelInfoArr *arr);
 static void get_rel_infos(migratorContext *ctx, const char *db_name,
-			  RelInfoArr *relarr, Cluster whichCluster);
+			const char *db_tblspace, RelInfoArr *relarr, Cluster whichCluster);
 static void relarr_free(RelInfoArr *rel_arr);
 static void map_rel(migratorContext *ctx, const RelInfo *oldrel,
 			const RelInfo *newrel, const DbInfo *old_db,
@@ -218,11 +218,18 @@ get_db_infos(migratorContext *ctx, DbInfoArr *dbinfs_arr, Cluster whichCluster)
 	DbInfo	   *dbinfos;
 	int			i_datname;
 	int			i_oid;
+	int			i_spclocation;
 
-	res = getDatabaseOIDs(conn);
-
+	res = executeQueryOrDie(ctx, conn, 
+				  "SELECT d.oid, d.datname, t.spclocation "
+				  "FROM pg_catalog.pg_database d "
+				  " LEFT OUTER JOIN pg_catalog.pg_tablespace t "
+				  " ON d.dattablespace = t.oid "
+				  "WHERE d.datname != 'template0'");
+				  
 	i_datname = PQfnumber(res, "datname");
 	i_oid = PQfnumber(res, "oid");
+	i_spclocation = PQfnumber(res, "spclocation");
 
 	ntups = PQntuples(res);
 	dbinfos = (DbInfo *) pg_malloc(ctx, sizeof(DbInfo) * ntups);
@@ -233,6 +240,8 @@ get_db_infos(migratorContext *ctx, DbInfoArr *dbinfs_arr, Cluster whichCluster)
 
 		snprintf(dbinfos[tupnum].db_name, sizeof(dbinfos[tupnum].db_name), "%s",
 				 PQgetvalue(res, tupnum, i_datname));
+		snprintf(dbinfos[tupnum].db_tblspace, sizeof(dbinfos[tupnum].db_tblspace), "%s",
+				 PQgetvalue(res, tupnum, i_spclocation));
 	}
 	PQclear(res);
 
@@ -258,6 +267,7 @@ get_db_and_rel_infos(migratorContext *ctx, DbInfoArr *db_arr, Cluster whichClust
 
 	for (dbnum = 0; dbnum < db_arr->ndbs; dbnum++)
 		get_rel_infos(ctx, db_arr->dbs[dbnum].db_name,
+					  db_arr->dbs[dbnum].db_tblspace,
 					  &(db_arr->dbs[dbnum].rel_arr), whichCluster);
 
 	if (ctx->debug)
@@ -275,7 +285,8 @@ get_db_and_rel_infos(migratorContext *ctx, DbInfoArr *db_arr, Cluster whichClust
  * FirstNormalObjectId belongs to the user
  */
 static void
-get_rel_infos(migratorContext *ctx, const char *db_name, RelInfoArr *relarr,
+get_rel_infos(migratorContext *ctx, const char *db_name,
+			  const char *db_tblspace, RelInfoArr *relarr,
 			  Cluster whichCluster)
 {
 	PGconn	   *conn = connectToServer(ctx, db_name, whichCluster);
@@ -307,8 +318,7 @@ get_rel_infos(migratorContext *ctx, const char *db_name, RelInfoArr *relarr,
 
 	snprintf(query, sizeof(query),
 			 "SELECT DISTINCT c.oid, n.nspname, c.relname, "
-			 "	c.relfilenode, c.reltoastrelid, "
-			 "	t.spclocation "
+			 "	c.relfilenode, c.reltoastrelid, t.spclocation "
 			 "FROM pg_catalog.pg_class c JOIN "
 			 "		pg_catalog.pg_namespace n "
 			 "	ON c.relnamespace = n.oid "
@@ -379,26 +389,25 @@ get_rel_infos(migratorContext *ctx, const char *db_name, RelInfoArr *relarr,
 
 	for (relnum = 0; relnum < ntups; relnum++)
 	{
-		RelInfo    *curr;
-		char	   *from = NULL;
-
-		nspname = PQgetvalue(res, relnum, i_nspname);
-		relname = PQgetvalue(res, relnum, i_relname);
-
-		curr = &relinfos[num_rels++];
-
-		from = PQgetvalue(res, relnum, i_spclocation);
-		snprintf(curr->tablespace, sizeof(curr->tablespace), "%s", from);
-
-		from = PQgetvalue(res, relnum, i_nspname);
+		RelInfo    *curr = &relinfos[num_rels++];
+		const char	   *tblspace;
 
 		curr->reloid = atol(PQgetvalue(res, relnum, i_oid));
 
+		nspname = PQgetvalue(res, relnum, i_nspname);
 		snprintf(curr->nspname, sizeof(curr->nspname), nspname);
+
+		relname = PQgetvalue(res, relnum, i_relname);
 		snprintf(curr->relname, sizeof(curr->relname), relname);
 
 		curr->relfilenode = atol(PQgetvalue(res, relnum, i_relfilenode));
 		curr->toastrelid = atol(PQgetvalue(res, relnum, i_reltoastrelid));
+
+		tblspace = PQgetvalue(res, relnum, i_spclocation);
+		/* if no table tablespace, use the database tablespace */
+		if (strlen(tblspace) == 0)
+			tblspace = db_tblspace;
+		snprintf(curr->tablespace, sizeof(curr->tablespace), "%s", tblspace);
 	}
 	PQclear(res);
 
