@@ -2,7 +2,7 @@
  *	pg_migrator.h
  */
 
-#define	PG_MIGRATOR_VERSION	"8.4.14"
+#define	PG_MIGRATOR_VERSION	"8.4.16"
 
 #include "postgres.h"
 
@@ -126,6 +126,7 @@ typedef struct
 {
 	Oid			db_oid;			/* oid of the database */
 	char		db_name[NAMEDATASIZE];	/* database name */
+	char		db_tblspace[MAXPGPATH]; /* database default tablespace path */
 	RelInfoArr	rel_arr;		/* array of all user relinfos */
 } DbInfo;
 
@@ -376,7 +377,6 @@ void		parseCommandLine(migratorContext *ctx, int argc, char *argv[]);
 
 void		reserve_relfilenodes(migratorContext *ctx);
 void		get_pg_database_relfilenode(migratorContext *ctx, Cluster whichCluster);
-PGresult   *getDatabaseOIDs(PGconn *conn);
 void		new_8_4_adjust_rels_to_use_fixed_toast_relfilenodes(migratorContext *ctx,
 			DbInfoArr *olddb_arr, DbInfoArr *newdb_arr);
 const char *transfer_all_new_dbs(migratorContext *ctx, DbInfoArr *olddb_arr,
